import java.util.*;
import java.awt.*;

/**
 * Esta clase representa aun fuente, vista como un sistema de particulas (o ms bien
 * un conjunto de sistemas de particulas), que intentan simular el comportamiente de
 * distintos chorros de agua.
 * @author Ricardo Monascal
 **/
public class Fuente {

// Constantes de la clase

// Campos de la clase

	/**
	 * Chorros de la fuente.
	 **/
	private LinkedList<Chorro> chorros;
	
	/**
	 * Color de esta fuente (indice en el arreglo de colores del mundo de juego).
	 **/
	private int color;

// Constructor de la clase.

	/**
	 * Constructor de la clase.
	 * El color default de la fuente es CYAN (Azul claro).
	 **/
	protected Fuente () {
		
		chorros = new LinkedList<Chorro>();
		color = 0;
		
	}

// Funciones y metodos de la clase.

	/**
	 * Agrega una chorro a esta fuente.
	 * @param   chorro   Chorro a agregar.
	 **/
	public void agregarChorro (Vector2D pos, Vector2D dir, double radio) {
		
		chorros.add(new Chorro (pos, dir, radio, color));
		
	}
	
	/**
	 * Cambia el color de todos los chorros asociados a esta fuente.
	 * @param   color   Nuevo color a asignar.
	 **/
	public void asignarColor (int color) {
		
		this.color = color;
		for (Chorro c : chorros)
			c.asignarColor(color);
		
	}
	
	/**
	 * Obtiene el color de esta fuente.
	 **/
	public int obtenerColor () {
		
		return color;
		
	}

	/**
	 * Actualiza esta fuente.
	 **/
	public void actualizar () {
		
		for (Chorro c : chorros)
			c.actualizar();
		
	}

	/**
	 * Pinta esta fuente.
	 * @param   g   Objeto grfico sobre el cual pintar.
	 **/
	public void pintar (Graphics g) {
		
		for (Chorro c : chorros)
			c.pintar(g);
		
	}

}