import java.util.*;
import java.awt.*;
import java.io.*;
import javax.sound.midi.*;

/**
 * Esta clase maneja el reproductor de msica del juego.
 * @author Ricardo Monascal
 **/
public class Reproductor {

// Constantes de la clase.

// Campos de la clase.

	/**
	 * Secuenciador del reproductor (quien se encarga de reproducir realmente).
	 **/
	private Sequencer reproductor;
	
	/**
	 * Nombre de la secuendia (Direccin en el sistema de archivos)
	 **/
	private String nombreSecuencia;
	
	/**
	 * Secuencia MIDI que se est reproduciondo
	 **/
	private Sequence secuencia;

// Constructor de la clase.

	/**
	 * Constructor de la clase.
	 **/
	protected Reproductor () {
		
		try {
			reproductor = MidiSystem.getSequencer();
			reproductor.open();
		}
		catch (MidiUnavailableException exc) {
			// No debera pasar.
		}
		
	}

// Funciones y mtodos de la clase.

	/**
	 * Devuelve la secuencia actual.
	 **/
	public Sequence obtenerSecuencia () {
		
		return secuencia;
		
	}

	/**
	 * Carga una secuencia MIDI a este reproductor.
	 * @param   nombre   Direccin completa del archivo MIDI a cargar
	 **/
	public void cargarSecuencia (String nombre) {
		
		nombreSecuencia = nombre;
		recargar();
		
	}

	/**
	 *Re carga la misma secuencia que ya tena.
	 **/
	public void recargar () {
		
		try {
			reproductor.stop();
			reproductor.close();
			secuencia = MidiSystem.getSequence(new File (nombreSecuencia));
			reproductor.open();
			reproductor.setSequence(secuencia);
		}
		catch (MidiUnavailableException exc) {
			// No debera pasar.
		}
		catch (InvalidMidiDataException exc) {
			// No debera pasar.
		}
		catch (IOException exc) {
			// No debera pasar.
		}
		
	}
	
	/**
	 * Pone a reproducir la secuencia que tenga cargado el reproductor
	 * (Siempre y cuando haya una secuancia cargada).
	 **/
	public void reproducir () {
		
		reproductor.start();
		
	}
	
	/**
	 * Termina de reproducir la secuencia.
	 **/
	public void terminar () {
		
		reproductor.stop();
		
	}
	
	/**
	 * Pausa la reproduccin la secuencia.
	 **/
	public void pausar () {
		
		reproductor.stop();
		
	}
	
	/**
	 * Agrega el listener de meta-eventos propuesto al reproductor.
	 * @param   listener   Listener a agregar.
	 **/
	public void agregarListener (MetaEventListener listener) {
		
		reproductor.addMetaEventListener(listener);
		
	}

}