import java.util.Random;
import java.awt.Point;

/**
 * Esta clase define funciones y metodos que comprenden
 * el manejo de vectores matematico de dos dimensiones.
 * @author Ricardo Monascal 
 **/
public class Vector2D {

// Constantes de la clase.

	/**
	 * Vector de magnitud cero.
	 **/
	public static final Vector2D CERO = new Vector2D ();

	/**
	 * Vector unitario que mira a la derecha.
	 **/
	public static final Vector2D DERECHA = new Vector2D (1, 0);

	/**
	 * Vector unitario que mira hacia arriba.
	 **/
	public static final Vector2D ARRIBA = new Vector2D (0, -1);

	/**
	 * Vector unitario que mira a la izquierda.
	 **/
	public static final Vector2D IZQUIERDA = new Vector2D (-1, 0);

	/**
	 * Vector unitario que mira hacia abajo.
	 **/
	public static final Vector2D ABAJO = new Vector2D (0, 1);

// Campos de la clase.

	/**
	 * Representacion de la primera posicion del Vector2D.
	 **/
	private double x;

	/**
	 * Representacion de la segunda posicion del Vector2D.
	 **/
	private double y;

// Constructor de la clase.

	/**
	 * Constructor de la clase.
	 **/
	public Vector2D (double x , double y) {
		
		this.x = x;
		this.y = y;
		
	}
	
	/**
	 * Constructor de la clase.
	 **/
	public Vector2D () {
		
		this(0,0);
		
	}
	
	/**
	 * Constructor de la clase.
	 **/
	public Vector2D (Vector2D v) {
		
		this(v.obtenerX(),v.obtenerY());
		
	}
	
	/**
	 * Constructor de la clase.
	 **/
	public Vector2D (Point p) {
		
		this(p.x,p.y);
		
	}

// Funciones de manipulacion externa de campos.

	/**
	 * Devuelve la primera posicion del Vector2D.
	 **/
	public double obtenerX () {
		
		return x;
		
	}

	/**
	 * Devuelve la segunda posicion del Vector2D.
	 **/
	public double obtenerY () {
		
		return y;
		
	}

	/**
	 * Reasigna la primera posicion del Vector2D.
	 **/
	public void asignarX (double x) {
		
		this.x = x;
		
	}

	/**
	 * Devuelve la segunda posicion del Vector2D.
	 **/
	public void asignarY (double y) {
		
		this.y = y;
		
	}

// Funciones y mtodos de la clase.

	/**
	 * Se asegura de que la primera posicin del Vector2D sea positiva.
	 **/
	public void positivoX () {
		
		x = Math.abs(x);
		
	}

	/**
	 * Se asegura de que la segunda posicin del Vector2D sea positiva.
	 **/
	public void positivoY () {
		
		y = Math.abs(y);
		
	}

	/**
	 * Se asegura de que la primera posicin del Vector2D sea negativa.
	 **/
	public void negativoX () {
		
		x = - Math.abs(x);
		
	}

	/**
	 * Se asegura de que la segunda posicin del Vector2D sea negativa.
	 **/
	public void negativoY () {
		
		y = - Math.abs(y);
		
	}

	/**
	 * Devuelve el negativo de este Vector2D.
	 **/
	public Vector2D neg () {
		
		return new Vector2D ( -x , -y );
		
	}

	/**
	 * Devuelve la suma de este Vector2D con otro nuevo.
	 **/
	public Vector2D sumar (Vector2D V) {
		
		return new Vector2D ( x + V.obtenerX() , y + V.obtenerY() );
		
	}

	/**
	 * Devuelve la resta de este Vector2D con otro nuevo.
	 **/
	public Vector2D restar (Vector2D V) {
		
		return sumar ( V.neg() );
		
	}

	/**
	 * Devuelve la multiplicacion de este Vector2D con un flotante.
	 **/
	public Vector2D mult (double d) {
		
		return new Vector2D ( x*d , y*d );
		
	}

	/**
	 * Devuelve un Vector2D perpendicular a este.
	 **/
	public Vector2D perp () {
		
		return new Vector2D ( -y , x );
		
	}

	/**
	 * Devuelve el Vector2D normalizado.
	 **/
	public Vector2D normalizar () {
		
		double magnitud = Math.sqrt( x*x + y*y );
		return new Vector2D ( x/magnitud , y/magnitud );	
		
	}

	/**
	 * Devuelve el producto producto punto Vector2D.
	 **/
	public double productoPunto (Vector2D V) {
		
		return x*V.obtenerX() + y*V.obtenerY();
		
	}

	/**
	 * Devuelve <code>1<\code> si el Vector propuesto esta en el sentido de las
	 * manillas del reloj con respecto al Vector original y cero de lo contrario.
	 **/
	public int signo (Vector2D V) {
		
		if ( y*V.obtenerX() > x*V.obtenerY() )
			return 1;
		else
			return -1;
		
	}

	/**
	 * Aplica al Vector2D modificaciones para poder tratar la ventana de
	 * juego como un toroide. Es decir, que es doblemente circular.
	 **/
	public void toroide (int maxX , int maxY) {
		
		if (x > maxX)
			x = 0.0;
		if (x < 0)
			x = (double) maxX;
		if (y < 0)
			y = (double) maxY;
		if (y > maxY)
			y = 0.0;
		
	}

	/**
	 *
	 **/
	public double modulo () {
		
		return Math.sqrt(x*x+y*y);
		
	}

	/**
	 *
	 **/
	public double modulo2 () {
		
		return x*x+y*y;
		
	}

	/**
	 *
	 **/
	public double distancia (Vector2D V) {
		
		return Math.sqrt(distancia2(V));
		
	}

	/**
	 *
	 **/
	public double distancia2 (Vector2D V) {
		
		double tempY = V.obtenerY()-y , tempX = V.obtenerX()-x;
		return tempY*tempY+tempX*tempX;
		
	}

	/**
	 *
	 **/
	public void truncar (double max) {
		
		Vector2D temp;
		if (modulo()>max) {
			temp = normalizar();
			x = temp.obtenerX()*max;
			y = temp.obtenerY()*max;
		}
			
	}

	/**
	 * Devuelve el angulo entre este vector y otro nuevo.
	 **/
	public double angulo (Vector2D V) {
		
		return Math.acos((this.normalizar().distancia2(V.normalizar())/2)-1);
		
	}

	/**
	 * Devuelve el vector que tiene el mismo modulo y va en direccion contraria a este.
	 **/
	public Vector2D inverso () {
		
		return new Vector2D (-x,-y);
		
	}
	
	/**
	 * Construye un nuevo Vector2D de direccin aleatoria, con
	 * la magnitud suministrada.
	 * @param   magnitud   Magnitud del Vector2D a crear.
	 **/
	public static Vector2D aleatorio (double magnitud) {
		
		double nx, ny;
		double eps = 1e-6; // Presiscin para comparar nulidad.
		
		// Hay que asegurarse que la magnitud no sea cero.
		do {
			nx = Math.random();
			ny = Math.random();
		} while (nx*nx + ny*ny < eps);
		
		return (new Vector2D (nx, ny)).normalizar().mult(magnitud);
		
	}

	/**
	 * Devuelve este Vector2D como un Point.
	 **/
	public Point aPoint () {
		
		return new Point ((int)x,(int)y);
		
	}

	/**
	 * Devuelve la representacion de este vector como un String.
	 **/
	public String toString() {
		return ("x: "+x+" y: "+y);
	}

}